<!-- ⚠️ DO NOT INCLUDE DATA/SCREENSHOTS THAT CAN'T BE PUBLICLY AVAILABLE. -->

<!-- You can erase any parts of this template not applicable to your report. -->

### 🐞 Bug report:

<!-- Remove this section if you are filling in a 💡 feature request. -->

**Describe the bug**
A clear and concise description of what the bug is.

**How to reproduce**
Steps to reproduce the behavior.

**Expected behavior**
A clear and concise description of what you expected to happen.

**Screenshots**
If applicable, add screenshots to help explain your problem.

---

### 💡 Feature request:

<!-- Remove this section if you are filling in a 🐞 bug report. -->

**Is your feature request related to a problem? Please describe**
A clear and concise description of what the problem is.

**Describe the solution you'd like**
A clear and concise description of what you want to happen.

**Describe alternatives you've considered**
A clear and concise description of any alternative solutions or features you've considered.

---

### 🔖 Version information

* **Plugin name:** `{{plugin_name}}`
* **Plugin project version:** `{{dist_name}}=={{dist_version}}`
* **Plugin project dependencies:**
    ```{% for dependency, version in dependencies.items() %}
    {{dependency}}=={{version}}
    {%- endfor %}      
    ```
